@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: timer.c,v 1.96 2004/11/30 16:11:04 mcr Exp $
+ * RCSID $Id: timer.c,v 1.99.2.1 2005/08/12 17:03:02 ken Exp $
  */
 
 #include <stdio.h>
@@ -157,7 +157,6 @@ handle_timer_event(void)
     struct event *ev = evlist;
     int type;
     struct state *st;
-    struct connection *c;
     ip_address peer;
 
     if (ev == (struct event *) NULL)    /* Just paranoid */
@@ -210,6 +209,7 @@ handle_timer_event(void)
     passert(GLOBALS_ARE_RESET());
     if (st != NULL)
     {
+	struct connection *c;
 	c = st->st_connection;
         if( type  == EVENT_DPD || type == EVENT_DPD_TIMEOUT)
         {
@@ -238,6 +238,12 @@ handle_timer_event(void)
 	    break;
 #endif
 
+        case EVENT_PENDING_PHASE2:
+	    passert(st == NULL);
+	    connection_check_phase2();
+	    break;
+	
+
 	case EVENT_LOG_DAILY:
 	    daily_log_event();
 	    break;
@@ -262,6 +268,10 @@ handle_timer_event(void)
 	     */
 	    {
 		time_t delay = 0;
+		struct connection *c;
+
+		passert(st != NULL);
+		c = st->st_connection;
 
 		DBG(DBG_CONTROL, DBG_log(
 		    "handling event EVENT_RETRANSMIT for %s \"%s\" #%lu"
@@ -284,7 +294,7 @@ handle_timer_event(void)
 			, "%s: retransmission; will wait %lus for response"
 			, enum_name(&state_names, st->st_state)
 			, (unsigned long)delay);
-		    send_packet(st, "EVENT_RETRANSMIT");
+		    send_packet(st, "EVENT_RETRANSMIT", TRUE);
 		    event_schedule(EVENT_RETRANSMIT, delay, st);
 		}
 		else
@@ -357,7 +367,12 @@ handle_timer_event(void)
 	case EVENT_SA_REPLACE:
 	case EVENT_SA_REPLACE_IF_USED:
 	    {
-		so_serial_t newest = IS_PHASE1(st->st_state)
+		struct connection *c;
+		so_serial_t newest;
+
+		passert(st != NULL);
+		c = st->st_connection;
+		newest = IS_PHASE1(st->st_state)
 		    ? c->newest_isakmp_sa : c->newest_ipsec_sa;
 
 		if (newest != st->st_serialno
@@ -410,6 +425,10 @@ handle_timer_event(void)
 	    {
 		const char *satype;
 		so_serial_t latest;
+		struct connection *c;
+
+		passert(st != NULL);
+		c = st->st_connection;
 
 		if (IS_PHASE1(st->st_state))
 		{
@@ -441,6 +460,10 @@ handle_timer_event(void)
 	    /* FALLTHROUGH */
 	case EVENT_SO_DISCARD:
 	    /* Delete this state object.  It must be in the hash table. */
+	    if(st->st_suspended_md) {
+		release_md(st->st_suspended_md);
+		st->st_suspended_md=NULL;
+	    }
 	    delete_state(st);
 	    break;
 
