@@ -8,8 +8,9 @@ TESTNAME=east-ah-icmp-01
 TESTHOST=east
 EXITONEMPTY=--exitonempty
 THREEEIGHT=true
-PRIVINPUT=../inputs/01-sunrise-sunset-ping.pcap
+PRIV_INPUT=../inputs/01-sunrise-sunset-ping.pcap
 REF_PUB_OUTPUT=ah1-output.txt
+REF26_PUB_OUTPUT=ah1-output26.txt
 REF_CONSOLE_OUTPUT=ah1-console.txt
 REF_CONSOLE_FIXUPS="kern-list-fixups.sed nocr.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS klips-spi-sanitize.sed"
